
# q-expansion of newform 1344.2.q.d, downloaded from the LMFDB on 10 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1344
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[127, [1, 0]], [1093, [1, 0]], [449, [1, 0]], [577, [0, -1]]]
aps_data = [[0, 0], [-1, 1], [0, -1], [1, -3], [-3, 3], [-4, 0], [0, 0], [0, 4], [0, 8], [3, 0], [-5, 5], [0, 8], [8, 0], [6, 0], [0, 10], [9, -9], [5, -5], [0, -10], [-6, 6], [-10, 0], [-2, 2], [0, 11], [7, 0], [0, 18], [-17, 0], [-2, 2], [0, 0], [0, 11], [-10, 10], [-8, 0], [7, 0], [0, -15], [14, -14], [-22, 0], [0, 18], [-5, 5], [-4, 4], [0, -8], [2, 0], [0, 2], [-12, 12], [0, 0], [0, -16], [27, -27], [26, 0], [12, -12], [6, 0], [-1, 0], [-21, 21], [0, -12], [0, 24], [0, 0], [-23, 23], [11, 0], [0, -14], [-14, 14], [1, -1], [0, -17], [8, -8], [6, 0], [18, -18], [19, 0], [-16, 0], [0, 0], [0, -9], [0, 13], [0, -4], [-15, 0], [20, -20], [-2, 0], [-8, 8], [0, 6], [-7, 7], [0, 16], [-16, 0], [0, 2], [26, -26], [0, 28], [0, 8], [-7, 7], [0, 0], [6, 0], [-12, 12], [-2, 0], [0, -9], [0, 1], [4, 0], [0, 17], [-30, 0], [16, 0], [0, 28], [6, -6], [37, -37], [-9, 0], [0, -10], [12, 0], [0, -39], [18, -18], [0, 12], [0, 34], [-8, 0], [15, -15], [-39, 39], [0, -12], [-22, 22], [33, -33], [45, 0], [0, 0], [-42, 42], [-13, 0], [0, 13], [8, -8], [-30, 0], [-6, 6], [11, 0], [22, -22], [46, 0], [-50, 50], [0, 15], [24, 0], [38, -38], [29, 0], [0, 19], [-17, 17], [0, 0], [-5, 0], [0, 6], [0, -34], [-47, 0], [0, 14], [-10, 10], [-34, 0], [0, -27], [10, 0], [0, 12], [37, 0], [-26, 26], [22, -22], [-3, 0], [16, -16], [2, 0], [0, -39], [32, -32], [-41, 0], [-4, 4], [24, 0], [-14, 0], [-14, 14], [0, 14], [0, 22], [0, -32], [-54, 0], [40, 0], [0, -48], [16, -16], [-6, 0], [0, -16], [0, -42], [19, 0], [-13, 13], [0, 48], [-38, 0], [-3, 0], [0, -25], [22, -22], [44, -44], [15, -15], [-26, 26], [15, 0], [0, -21], [19, -19], [18, 0], [0, 4], [27, -27], [-11, 11], [-4, 0], [-46, 0], [-45, 45], [24, 0], [0, -6], [0, -37], [-51, 0], [26, 0], [0, -14], [26, -26], [31, -31], [-26, 26], [-42, 42], [0, -59], [-20, 20], [0, 31], [21, 0], [0, -52], [0, 58], [-7, 7], [-18, 18], [26, -26], [0, -66], [62, 0], [0, -22], [-1, 1], [-64, 0], [0, -42], [38, -38], [16, 0], [69, -69], [0, 24], [0, -21], [56, 0], [34, -34], [0, 62], [-11, 0], [-35, 0], [0, -61], [28, -28], [0, -18], [1, -1], [-56, 56], [0, 42], [2, 0], [0, 8], [64, 0], [0, -58], [0, 31], [33, 0], [-64, 0], [0, -26], [28, -28], [0, -9], [0, -12], [-6, 6], [8, -8], [-51, 0], [-30, 30], [-2, 0], [-60, 60], [0, 13], [0, -21], [-68, 0], [16, 0], [-60, 60], [0, -16], [17, -17], [0, 50], [36, 0], [0, 12], [-19, 0], [36, -36], [16, -16], [64, 0], [-26, 0], [0, 78], [6, -6], [18, 0], [49, -49], [0, 21], [-30, 30], [28, -28], [-1, 0], [0, 67], [40, -40], [15, 0], [54, -54], [76, 0], [-2, 2], [0, 70], [5, 0], [60, 0], [20, 0], [-29, 29], [0, -48], [10, -10], [59, -59], [0, -51], [-43, 0], [0, -56], [0, -29], [-40, 40], [0, -54], [0, 32], [-22, 22], [-17, 17], [-36, 0], [36, 0], [0, -50], [0, -8], [-79, 79], [45, 0], [53, -53], [84, 0], [10, -10], [-53, 53], [0, -12], [20, 0], [-4, 0], [3, -3], [0, 56], [86, 0], [0, 60], [-62, 0], [0, -6], [0, 11], [-59, 59]]
